#!/usr/bin/perl

use strict;
use warnings;
use utf8;
use File::Path;



###
### Const variable
###
my $OPT_DEBUG   = 0;
my $OPT_FLAG    = 0;
my $OPT_HELP    = 0;
my $OPT_WEB     = 0;
my $OPT_RETURN  = 1;
my @OPT_ARGS;

my $DATE        = `date "+%Y%m%d"`;
chop($DATE);
my $FILE_IPV4   = "CIDR-IPV4";
my $FILE_IPV6   = "CIDR-IPV6";
my $FILE_MAIL   = "CIDR-MAIL";

my $DIFF_LIMIT  = 500;
my @DIFF_RESULT_IPV4;
my @DIFF_RESULT_IPV6;
my @DIFF_RESULT_MAIL;

my %COUNT_IPV4;
$COUNT_IPV4{'ADD'}  = 0;
$COUNT_IPV4{'DEL'}  = 0;
$COUNT_IPV4{'SUM'}  = 0;

my %COUNT_IPV6;
$COUNT_IPV6{'ADD'}  = 0;
$COUNT_IPV6{'DEL'}  = 0;
$COUNT_IPV6{'SUM'}  = 0;

my $CIDR_ROOT_DIR   = "/var/iplist";
my $CIDR_TODAY_DIR  = "$CIDR_ROOT_DIR/$DATE";

my $MAIN_TITLE      = "Country ip list generate report \[$DATE\]";
my $MAIL_FROM       = 'testmail-from\@gmail.com';
my $MAIL_TO         = 'testmail-to\@gmail.com';
my $MAIL_FLAG       = 0;

my $BIN_CAT         = "/usr/bin/cat";
my $BIN_CP          = "/usr/bin/cp";
my $BIN_DIFF        = "/usr/bin/diff";
my $BIN_FIND        = "/usr/bin/find";
my $BIN_MAIL        = "/usr/bin/mail";
my $BIN_TAR         = "/bin/tar";
my $BIN_TOUCH       = "/usr/bin/touch";

my $SCRIPT_ROOT_DIR = "/usr/local/bin/iplist";
my $SCRIPT_OPTION   = "";
my $SCRIPT_GETRIR   = "$SCRIPT_ROOT_DIR/make-country-getrir.pl";
my $SCRIPT_MODIFY   = "$SCRIPT_ROOT_DIR/make-country-modify.pl";
my $SCRIPT_OUTPUT   = "$SCRIPT_ROOT_DIR/make-country-output.pl";
my $SCRIPT_WEBDIR   = "$SCRIPT_ROOT_DIR/make-country-webdir.pl";



###
### Subroutine
###
## Update current IPv4 ip list
##
sub CHECK_UPDATE_IPV4{
    if((-f "$CIDR_ROOT_DIR/$FILE_IPV4")&&(-f "$CIDR_TODAY_DIR/$FILE_IPV4")){
        if($OPT_DEBUG){
            print "VALUE::CIDR_NOW_FILE   $CIDR_ROOT_DIR/$FILE_IPV4\n";
            print "VALUE::CIDR_TODAY_FILE $CIDR_TODAY_DIR/$FILE_IPV4\n";
            print "-----\n";
        }

        # Acquire ip list diff and count rows
        foreach my $tmp_LINE(`$BIN_DIFF -U 1 $CIDR_ROOT_DIR/$FILE_IPV4 $CIDR_TODAY_DIR/$FILE_IPV4`){
            chop($tmp_LINE);
            push(@DIFF_RESULT_IPV4, "$tmp_LINE");
            if($tmp_LINE =~ /^(\+)[a-zA-Z0-9]+/){ $COUNT_IPV4{'ADD'}++; }
            if($tmp_LINE =~ /^(\-)[a-zA-Z0-9]+/){ $COUNT_IPV4{'DEL'}++; }
        }
        $COUNT_IPV4{'SUM'} = $COUNT_IPV4{'ADD'} + $COUNT_IPV4{'DEL'};

        # Threshold confirmation of diff line number
        if(($COUNT_IPV4{'SUM'} < $DIFF_LIMIT)||($DIFF_LIMIT == 0)){
            if($OPT_DEBUG){
                print "EXEC::REMOVE $CIDR_ROOT_DIR/$FILE_IPV4\n";
                print "EXEC::COPY   $CIDR_TODAY_DIR/$FILE_IPV4 $CIDR_ROOT_DIR/$FILE_IPV4\n";
                print "-----\n";
            }
            unlink "$CIDR_ROOT_DIR/$FILE_IPV4";
            system("$BIN_CP $CIDR_TODAY_DIR/$FILE_IPV4 $CIDR_ROOT_DIR/$FILE_IPV4");
        }else{
            if($OPT_DEBUG){
                print "VALUE::COUNT_IPV4{ADD} $COUNT_IPV4{'ADD'}\n";
                print "VALUE::COUNT_IPV4{DEL} $COUNT_IPV4{'DEL'}\n";
                print "VALUE::COUNT_IPV4{SUM} $COUNT_IPV4{'SUM'}\n";
                print "-----\n";
            }
            @DIFF_RESULT_IPV4 = ();
            push(@DIFF_RESULT_IPV4, " Difference current file and create file was $COUNT_IPV4{'SUM'} line.");
            push(@DIFF_RESULT_IPV4, " Does not update ip list. Because exceeds the threshold of $DIFF_LIMIT.");
        }
    }
    return 0;
}


## Update current IPv6 ip list
##
sub CHECK_UPDATE_IPV6{
    if((-f "$CIDR_ROOT_DIR/$FILE_IPV6")&&(-f "$CIDR_TODAY_DIR/$FILE_IPV6")){
        if($OPT_DEBUG){
            print "VALUE::CIDR_NOW_FILE   $CIDR_ROOT_DIR/$FILE_IPV6\n";
            print "VALUE::CIDR_TODAY_FILE $CIDR_TODAY_DIR/$FILE_IPV6\n";
            print "-----\n";
        }

        # Acquire ip list diff and count rows
        foreach my $tmp_LINE(`$BIN_DIFF -U 1 $CIDR_ROOT_DIR/$FILE_IPV6 $CIDR_TODAY_DIR/$FILE_IPV6`){
            chop($tmp_LINE);
            push(@DIFF_RESULT_IPV6, "$tmp_LINE");
            if($tmp_LINE =~ /^(\+)[a-zA-Z0-9]+/){ $COUNT_IPV6{'ADD'}++; }
            if($tmp_LINE =~ /^(\-)[a-zA-Z0-9]+/){ $COUNT_IPV6{'DEL'}++; }
        }
        $COUNT_IPV6{'SUM'} = $COUNT_IPV6{'ADD'} + $COUNT_IPV6{'DEL'};

        # Threshold confirmation of diff line number
        if(($COUNT_IPV6{'SUM'} < $DIFF_LIMIT)||($DIFF_LIMIT == 0)){
            if($OPT_DEBUG){
                print "EXEC::REMOVE $CIDR_ROOT_DIR/$FILE_IPV6\n";
                print "EXEC::COPY   $CIDR_TODAY_DIR/$FILE_IPV6 $CIDR_ROOT_DIR/$FILE_IPV6\n";
                print "-----\n";
            }
            unlink "$CIDR_ROOT_DIR/$FILE_IPV6";
            system("$BIN_CP $CIDR_TODAY_DIR/$FILE_IPV6 $CIDR_ROOT_DIR/$FILE_IPV6");
        }else{
            if($OPT_DEBUG){
                print "VALUE::COUNT_IPV6{ADD} $COUNT_IPV6{'ADD'}\n";
                print "VALUE::COUNT_IPV6{DEL} $COUNT_IPV6{'DEL'}\n";
                print "VALUE::COUNT_IPV6{SUM} $COUNT_IPV6{'SUM'}\n";
                print "-----\n";
            }
            @DIFF_RESULT_IPV6 = ();
            push(@DIFF_RESULT_IPV6, " Difference current file and create file was $COUNT_IPV6{'SUM'} line.");
            push(@DIFF_RESULT_IPV6, " Does not update ip list. Because exceeds the threshold of $DIFF_LIMIT.");
        }
    }
    return 0;
}


## Send file create result to administrator
##
sub CREATE_REPORT_MAIL{
    my $tmp_CHAR;
    my $tmp_LENGTH;

    # Mail header text
    push(@DIFF_RESULT_MAIL, "Change history of ip address list by country.\n");
    push(@DIFF_RESULT_MAIL, "Check the difference in the execution result.\n");
    push(@DIFF_RESULT_MAIL, "\n");
    push(@DIFF_RESULT_MAIL, "      | + , Add | - , Del | = , Sum\n");
    push(@DIFF_RESULT_MAIL, " ----------------------------------\n");

    # IPv4 change counter
    $tmp_CHAR = " IPv4";
    foreach my $tmp_KEY(sort keys(%COUNT_IPV4)){
        $tmp_CHAR = $tmp_CHAR . ' | ';
        for(my $tmp_COUNT=0; $tmp_COUNT<abs(7-length($COUNT_IPV4{$tmp_KEY})); $tmp_COUNT++){
            $tmp_CHAR = $tmp_CHAR . ' ';
        }
        $tmp_CHAR = $tmp_CHAR . $COUNT_IPV4{$tmp_KEY};
    }
    push(@DIFF_RESULT_MAIL, "$tmp_CHAR\n");

    # IPv6 change counter
    $tmp_CHAR = " IPv6";
    foreach my $tmp_KEY(sort keys(%COUNT_IPV6)){
        $tmp_CHAR = $tmp_CHAR . ' | ';
        for(my $tmp_COUNT=0; $tmp_COUNT<abs(7-length($COUNT_IPV6{$tmp_KEY})); $tmp_COUNT++){
            $tmp_CHAR = $tmp_CHAR . ' ';
        }
        $tmp_CHAR = $tmp_CHAR . $COUNT_IPV6{$tmp_KEY};
    }
    push(@DIFF_RESULT_MAIL, "$tmp_CHAR\n");

    # IPv4 country ip list
    $tmp_LENGTH = @DIFF_RESULT_IPV4;
    if($tmp_LENGTH > 0){
        $MAIL_FLAG++;
        push(@DIFF_RESULT_MAIL, "\n");
        push(@DIFF_RESULT_MAIL, "==== ==== ==== ==== ==== ==== ==== ==== ====\n");
        push(@DIFF_RESULT_MAIL, "Difference result of IPv4 list by country   \n");
        push(@DIFF_RESULT_MAIL, "==== ==== ==== ==== ==== ==== ==== ==== ====\n");
        foreach my $tmp_LINE(@DIFF_RESULT_IPV4){
            push(@DIFF_RESULT_MAIL, "$tmp_LINE\n");
        }
    }

    # IPv6 country ip list
    $tmp_LENGTH = @DIFF_RESULT_IPV6;
    if($tmp_LENGTH > 0){
        $MAIL_FLAG++;
        push(@DIFF_RESULT_MAIL, "\n");
        push(@DIFF_RESULT_MAIL, "==== ==== ==== ==== ==== ==== ==== ==== ====\n");
        push(@DIFF_RESULT_MAIL, "Difference result of IPv6 list by country   \n");
        push(@DIFF_RESULT_MAIL, "==== ==== ==== ==== ==== ==== ==== ==== ====\n");
        foreach my $tmp_LINE(@DIFF_RESULT_IPV6){
            push(@DIFF_RESULT_MAIL, "$tmp_LINE\n");
        }
    }

    # Write mail of text file
    open(WRITE_MAIL, ">> $CIDR_TODAY_DIR/$FILE_MAIL");
    foreach my $tmp_LINE(@DIFF_RESULT_MAIL){
        if($OPT_DEBUG){ print "VALUE::MAIL $tmp_LINE"; }
        print WRITE_MAIL "$tmp_LINE";
    }
    close(WRITE_MAIL);
    return 0;
}


## Send file create report to the administrator by mail
##
sub SEND_REPORT_MAIL{
    if($OPT_DEBUG){
        print "VALUE::MAIL_TITLE $MAIN_TITLE\n";
        print "VALUE::MAIL_FROM  $MAIL_FROM\n";
        print "VALUE::MAIL_TO    $MAIL_TO\n";
        print "VALUE::FILE_MAIL  $CIDR_TODAY_DIR/$FILE_MAIL";
        print "-----\n";
    }else{
        if($MAIL_FLAG){
            system("$BIN_CAT $CIDR_TODAY_DIR/$FILE_MAIL | $BIN_MAIL -s '$MAIN_TITLE' -r $MAIL_FROM $MAIL_TO");
        }
    }
    return 0;
}


## Compress of today directory
##
sub COMPRESS_TODAY_IPLIST{
    if(-e $CIDR_ROOT_DIR){
        chdir $CIDR_ROOT_DIR;
        foreach my $tmp_PATH(`$BIN_FIND ./ -maxdepth 1 -type d`){
            chop($tmp_PATH);
            $tmp_PATH =~ s/\.\///g;
            if($OPT_DEBUG){
                print "VALUE::PATH $tmp_PATH\n";
                print "-----\n";
            }
            if($tmp_PATH =~ /^20[0-9]+$/){
                if(-e "$tmp_PATH"){
                    system("$BIN_TAR Jcf $tmp_PATH.tar.xz $tmp_PATH");
                    rmtree("$tmp_PATH");
                }
            }
        }
    }
    return 0;
}


## Output help
##
sub OUTPUT_SCRIPT_HELP{
    print 'Usage: make-country-iplist.pl -l OBJECT [ OPTIONS ]'."\n";
    print '                                                   '."\n";
    print 'OBJECT  := LIMIT       / Difference threshold      '."\n";
    print '                         Force exec when 0 is set  '."\n";
    print 'OPTIONS := -d[--debug] / Standard stream for debug '."\n";
    print '           -h[--help]  / Output of this help       '."\n";
    print '           -w[--web]   / Copy file to web directory'."\n";
    print '                                                   '."\n";
    print 'Example of execution:                              '."\n";
    print '  $ ./make-country-iplist.pl                       '."\n";
    print '  $ ./make-country-iplist.pl -l LIMIT              '."\n";
    print '  $ ./make-country-iplist.pl -l LIMIT --web        '."\n";
    return 0;
}



###
### Main Program
###
## Setting options and arguments
##
# Set of execution options
foreach my $tmp_OPT(@ARGV){
    push(@OPT_ARGS, $tmp_OPT);
    if(($tmp_OPT eq '-d')||($tmp_OPT eq '--debug')){ $OPT_DEBUG++; }
    if(($tmp_OPT eq '-h')||($tmp_OPT eq '--help' )){ $OPT_HELP++;  }
    if(($tmp_OPT eq '-w')||($tmp_OPT eq '--web'  )){ $OPT_WEB++;   }
}
if($OPT_DEBUG){
    print "VALUE::OPT_DEBUG $OPT_DEBUG\n";
    print "VALUE::OPT_HELP  $OPT_HELP\n";
    print "VALUE::OPT_WEB   $OPT_WEB\n";
    print "-----\n";
}

# Setting difference threshold limit
for(my $tmp_COUNT=0; $tmp_COUNT<($#OPT_ARGS+1); $tmp_COUNT++){
    if($OPT_ARGS[$tmp_COUNT] eq '-l'){ $DIFF_LIMIT = $OPT_ARGS[$tmp_COUNT+1]; }
}
if($OPT_DEBUG){
    print "VALUE::DIFF_LIMIT $DIFF_LIMIT\n";
    print "-----\n";
}

# Execute of initial process or help
if($OPT_HELP){
    $OPT_RETURN = &OUTPUT_SCRIPT_HELP;
}else{
    if(-e "$CIDR_TODAY_DIR"              ){ rmtree("$CIDR_TODAY_DIR");                      }
    if(-f "$CIDR_ROOT_DIR/$DATE.tar.xz"  ){ unlink "$CIDR_ROOT_DIR/$DATE.tar.xz";           }
    unless(-f "$CIDR_ROOT_DIR/$FILE_IPV4"){ system("$BIN_TOUCH $CIDR_ROOT_DIR/$FILE_IPV4"); }
    unless(-f "$CIDR_ROOT_DIR/$FILE_IPV6"){ system("$BIN_TOUCH $CIDR_ROOT_DIR/$FILE_IPV6"); }

    # Get country ip list and duplicate delete and modify
    unless($OPT_FLAG){ $OPT_FLAG = system("$SCRIPT_GETRIR -s $CIDR_ROOT_DIR $SCRIPT_OPTION"); }
    unless($OPT_FLAG){
        foreach my $tmp_FILE(glob("$CIDR_TODAY_DIR/RIR/*.IP")){
            $OPT_FLAG = system("$SCRIPT_MODIFY -f $tmp_FILE $SCRIPT_OPTION");
        }
    }

    # Output of country ip list
    unless($OPT_FLAG){ $OPT_FLAG = system("$SCRIPT_OUTPUT -f4 $FILE_IPV4 -f6 $FILE_IPV6 $SCRIPT_OPTION"); }

    # Check with current ip list
    unless($OPT_FLAG){ $OPT_FLAG = &CHECK_UPDATE_IPV4; }
    unless($OPT_FLAG){ $OPT_FLAG = &CHECK_UPDATE_IPV6; }

    # Report create and mail send
    unless($OPT_FLAG){ $OPT_FLAG = &CREATE_REPORT_MAIL; }
    unless($OPT_FLAG){ $OPT_FLAG = &SEND_REPORT_MAIL;   }

    # Compress generated today file and copy to WEB
    unless($OPT_FLAG){ $OPT_FLAG = &COMPRESS_TODAY_IPLIST; }
    unless($OPT_FLAG){
        if($OPT_WEB ){ $OPT_FLAG = system("$SCRIPT_WEBDIR $SCRIPT_OPTION"); }
    }
    unless($OPT_FLAG){ $OPT_RETURN = 0; }
}

exit $OPT_RETURN;