#!/usr/bin/perl

use strict;
use warnings;
use utf8;
use Net::IP;



###
### Const variable
###
my $OPT_DEBUG   = 0;
my $OPT_CLEAR   = 0;
my $OPT_FLAG    = 0;
my $OPT_HELP    = 0;
my $OPT_RETURN  = 1;
my @OPT_ARGS;

my $BIN_MKDIR   = "/usr/bin/mkdir";
my $DATE        = `date "+%Y%m%d"`;
chop($DATE);
my %IP_IPV4;
my %IP_IPV6;

my $CIDR_LIST_FILE  = "";
my $CIDR_ROOT_DIR   = "/var/iplist";
my $CIDR_IPV4_DIR   = "$CIDR_ROOT_DIR/$DATE/IPV4";
my $CIDR_IPV6_DIR   = "$CIDR_ROOT_DIR/$DATE/IPV6";



###
### Subroutine
###
## To correct processing of country ip list
##
sub MODIFY_COUNTRY_IPLIST{
    my $tmp_CC;
    my $tmp_TYPE;
    my $tmp_START;
    my $tmp_VALUE;

    # Open ip list file
    open(READ_IPLIST, "$CIDR_LIST_FILE");
    foreach(<READ_IPLIST>){
        chop;
        if($_ =~ /\|(..)\|ipv([46])\|([^\|]+)\|(\d+)\|/){
            $tmp_CC    = $1;
            $tmp_TYPE  = $2;
            $tmp_START = $3;
            $tmp_VALUE = $4;

            # Processing for IPv4
            if($tmp_TYPE == 4){
                if(!exists($IP_IPV4{$tmp_CC})){
                    my @key_CC;
                    $IP_IPV4{$tmp_CC} = \@key_CC;
                }
                my $IP = new Net::IP("$tmp_START + ".("$tmp_VALUE" - 1));
                foreach($IP->find_prefixes()){
                    push($IP_IPV4{$tmp_CC}, "$_");
                }
            }

            # Processing for IPv6
            if($tmp_TYPE == 6){
                if(!exists($IP_IPV6{$tmp_CC})){
                    my @key_CC;
                    $IP_IPV6{$tmp_CC} = \@key_CC;
                }
                my $IP = new Net::IP("$tmp_START/$tmp_VALUE");
                foreach($IP->prefix()){
                    push($IP_IPV6{$tmp_CC}, "$_");
                }
            }
        }
    }
    return 0;
}


## Write a formatted IPv4
##
sub WRITE_FORMAT_IPV4{
    foreach my $tmp_CC(sort keys(%IP_IPV4)){
        foreach my $tmp_LIST($IP_IPV4{$tmp_CC}){
            open(WRITE_IPLIST, ">> $CIDR_IPV4_DIR/$tmp_CC");
            foreach my $tmp_IP(@$tmp_LIST){
                if($OPT_DEBUG){ print "VALUE::IPLIST $tmp_CC $tmp_IP\n"; }
                print WRITE_IPLIST "$tmp_IP\n";
            }
            close(WRITE_IPLIST);
            undef(@$tmp_LIST);
        }
    }
    undef(%IP_IPV4);
    return 0;
}


## Write a formatted IPv6
##
sub WRITE_FORMAT_IPV6{
    foreach my $tmp_CC(sort keys(%IP_IPV6)){
        foreach my $tmp_LIST($IP_IPV6{$tmp_CC}){
            open(WRITE_IPLIST, ">> $CIDR_IPV6_DIR/$tmp_CC");
            foreach my $tmp_IP(@$tmp_LIST){
                if($OPT_DEBUG){ print "VALUE::IPLIST $tmp_CC $tmp_IP\n"; }
                print WRITE_IPLIST "$tmp_IP\n";
            }
            close(WRITE_IPLIST);
            undef(@$tmp_LIST);
        }
    }
    undef(%IP_IPV6);
    return 0;
}


## Output help
##
sub OUTPUT_SCRIPT_HELP{
    print 'Usage: make-country-modify.pl -f OBJECT [ OPTIONS ]'."\n";
    print '                                                   '."\n";
    print 'OBJECT  := FILEPATH    / Read country ip list file '."\n";
    print 'OPTIONS := -c[--clear] / Delete created ip list    '."\n";
    print '           -d[--debug] / Standard stream for debug '."\n";
    print '           -h[--help]  / Output of this help       '."\n";
    print '                                                   '."\n";
    print 'Example of execution:                              '."\n";
    print '  $ ./make-country-modify.pl -f FILEPATH           '."\n";
    print '  $ ./make-country-modify.pl -f FILEPATH --clear   '."\n";
    print '  $ ./make-country-modify.pl -f FILEPATH --debug   '."\n";
    return 0;
}



###
### Main Program
###
## Setting options and arguments
##
# Set of execution options
foreach my $tmp_OPT(@ARGV){
    push(@OPT_ARGS, $tmp_OPT);
    if(($tmp_OPT eq '-c')||($tmp_OPT eq '--clear')){ $OPT_CLEAR++; }
    if(($tmp_OPT eq '-d')||($tmp_OPT eq '--debug')){ $OPT_DEBUG++; }
    if(($tmp_OPT eq '-h')||($tmp_OPT eq '--help' )){ $OPT_HELP++;  }
}
if($OPT_DEBUG){
    print "VALUE::OPT_CLEAR $OPT_CLEAR\n";
    print "VALUE::OPT_DEBUG $OPT_DEBUG\n";
    print "VALUE::OPT_HELP  $OPT_HELP\n";
    print "-----\n";
}

# Set of read ip list
for(my $tmp_COUNT=0; $tmp_COUNT<($#OPT_ARGS+1); $tmp_COUNT++){
    if($OPT_ARGS[$tmp_COUNT] eq '-f'){ $CIDR_LIST_FILE = $OPT_ARGS[$tmp_COUNT+1]; }
}
if($OPT_DEBUG){
    print "VALUE::CIDR_LIST_FILE $CIDR_LIST_FILE\n";
    print "-----\n";
}

# Execute of initial process or help
if($OPT_HELP){
    $OPT_RETURN = &OUTPUT_SCRIPT_HELP;
}else{
    unless(-e "$CIDR_IPV4_DIR"){ system("$BIN_MKDIR -p $CIDR_IPV4_DIR"); }
    unless(-e "$CIDR_IPV6_DIR"){ system("$BIN_MKDIR -p $CIDR_IPV6_DIR"); }
    if($OPT_CLEAR){
        foreach my $tmp_FILE(glob("$CIDR_IPV4_DIR/*")){
            if(-f "$tmp_FILE"){ unlink "$tmp_FILE"; }
        }
        foreach my $tmp_FILE(glob("$CIDR_IPV6_DIR/*")){
            if(-f "$tmp_FILE"){ unlink "$tmp_FILE"; }
        }
    }
    if(-e "$CIDR_LIST_FILE"){
        unless($OPT_FLAG){ $OPT_FLAG = &MODIFY_COUNTRY_IPLIST; }
        unless($OPT_FLAG){ $OPT_FLAG = &WRITE_FORMAT_IPV4; }
        unless($OPT_FLAG){ $OPT_FLAG = &WRITE_FORMAT_IPV6; }
        unless($OPT_FLAG){ $OPT_RETURN = 0; }
    }
}

exit $OPT_RETURN;