#!/usr/bin/perl

use strict;
use warnings;
use utf8;



###
### Const variable
###
my $OPT_DEBUG   = 0;
my $OPT_AGGR    = 0;
my $OPT_FLAG    = 0;
my $OPT_HELP    = 0;
my $OPT_RETURN  = 1;
my @OPT_ARGS;

my $BIN_CAT     = "/usr/bin/cat";
my $DATE        = `date "+%Y%m%d"`;
chop($DATE);
my $FILE_IPV4   = "CIDR-IPV4";
my $FILE_IPV6   = "CIDR-IPV6";

my $CIDR_SCRIPT     = "/usr/local/bin/iplist/aggregate-cidr-addresses";
my $CIDR_ROOT_DIR   = "/var/iplist";
my $CIDR_IPV4_DIR   = "$CIDR_ROOT_DIR/$DATE/IPV4";
my $CIDR_IPV6_DIR   = "$CIDR_ROOT_DIR/$DATE/IPV6";



###
### Subroutine
###
## Aggregate IPv4 segments and write file
##
sub WRITE_AGGREGATE_IPV4{
    foreach my $tmp_FILE(glob("$CIDR_IPV4_DIR/*")){
        my (@BEFORE, @AFTER);
        my $tmp_CC =  $tmp_FILE;
           $tmp_CC =~ s/.*\///;

        # IP address aggregate
        @AFTER = `$BIN_CAT $tmp_FILE | $CIDR_SCRIPT`;

        # Write of formatted IP address
        open(WRITE_IPLIST, ">> $CIDR_ROOT_DIR/$DATE/$FILE_IPV4");
        foreach my $tmp_IP(@AFTER){
            chop($tmp_IP);
            if($OPT_DEBUG){
                print "VALUE::IPLIST $tmp_CC $tmp_IP\n";
            }else{
                print WRITE_IPLIST "$tmp_CC\t$tmp_IP\n";
            }
        }
        close(WRITE_IPLIST);
    }
    return 0;
}


## Aggregate IPv6 segments and write file
##
sub WRITE_AGGREGATE_IPV6{
    foreach my $tmp_FILE(glob("$CIDR_IPV6_DIR/*")){
        my (@BEFORE, @AFTER);
        my $tmp_CC =  $tmp_FILE;
           $tmp_CC =~ s/.*\///;

        # IP address aggregate
        @AFTER = `$BIN_CAT $tmp_FILE | $CIDR_SCRIPT`;

        # Write of formatted IP address
        open(WRITE_IPLIST, ">> $CIDR_ROOT_DIR/$DATE/$FILE_IPV6");
        foreach my $tmp_IP(@AFTER){
            chop($tmp_IP);
            if($OPT_DEBUG){
                print "VALUE::IPLIST $tmp_CC $tmp_IP\n";
            }else{
                print WRITE_IPLIST "$tmp_CC\t$tmp_IP\n";
            }
        }
        close(WRITE_IPLIST);
    }
    return 0;
}


## Sort IPv4 address in ascending order
##
sub SORT_ASCENDING_IPV4{
    foreach my $tmp_FILE(glob("$CIDR_IPV4_DIR/*")){
        if(-f $tmp_FILE){
            my (@BEFORE, @AFTER, @UNIQ, %FLAG);
            open(READ_IPLIST, "$tmp_FILE");
            foreach(<READ_IPLIST>){
                chop;
                push(@BEFORE, "$_");
            }
            close(READ_IPLIST);

            # Sort in ascend order and duplicate delete of ip
            @AFTER = sort {
                my @a = split(/\.|\//, $a);
                my @b = split(/\.|\//, $b);
                    $a[0] <=> $b[0] ||
                    $a[1] <=> $b[1] ||
                    $a[2] <=> $b[2] ||
                    $a[3] <=> $b[3] ||
                    $a[4] <=> $b[4]
            } @BEFORE;
            @UNIQ = grep { $FLAG{$_}++ < 1; } @AFTER;

            # Overwrite sorted
            unless($OPT_DEBUG){ unlink "$tmp_FILE"; }
            open(WRITE_IPLIST, ">> $tmp_FILE");
            foreach my $tmp_IP(@UNIQ){
                if($OPT_DEBUG){
                    print "VALUE::IPLIST $tmp_IP\n";
                }else{
                    print WRITE_IPLIST "$tmp_IP\n";
                }
            }
            close(WRITE_IPLIST);
        }
    }
    return 0;
}


## Sort IPv6 address in ascending order
##
sub SORT_ASCENDING_IPV6{
    foreach my $tmp_FILE(glob("$CIDR_IPV6_DIR/*")){
        if(-f $tmp_FILE){
            my (@BEFORE, @AFTER, @UNIQ, %FLAG);
            open(READ_IPLIST, "$tmp_FILE");
            foreach(<READ_IPLIST>){
                chop;
                push(@BEFORE, "$_");
            }
            close(READ_IPLIST);

            # Sort in ascend order and duplicate delete of ip
            @AFTER = sort {
                my @a = split(/\:|\//, $a);
                my @b = split(/\:|\//, $b);
                    $a[0] cmp $b[0] ||
                    $a[1] cmp $b[1] ||
                    $a[2] cmp $b[2] ||
                    $a[3] cmp $b[3] ||
                    $a[4] cmp $b[4] ||
                    $a[5] cmp $b[5] ||
                    $a[6] cmp $b[6] ||
                    $a[7] cmp $b[7] ||
                    $a[8] <=> $b[8]
            } @BEFORE;
            @UNIQ = grep { $FLAG{$_}++ < 1; } @AFTER;

            # Overwrite sorted
            unless($OPT_DEBUG){ unlink "$tmp_FILE"; }
            open(WRITE_IPLIST, ">> $tmp_FILE");
            foreach my $tmp_IP(@UNIQ){
                if($OPT_DEBUG){
                    print "VALUE::IPLIST $tmp_IP\n";
                }else{
                    print WRITE_IPLIST "$tmp_IP\n";
                }
            }
            close(WRITE_IPLIST);
        }
    }
    return 0;
}


## Output help
##
sub OUTPUT_SCRIPT_HELP{
    print 'Usage: make-country-output.pl -f4 OBJECT -f6 OBJECT [ OPTIONS ]'."\n";
    print '                                                               '."\n";
    print 'OBJECT  := FILENAME        / IP list file name                 '."\n";
    print 'OPTIONS := -a[--aggregate] / Process duplicate delete and sort '."\n";
    print '           -d[--debug]     / Standard stream for debug         '."\n";
    print '           -h[--help]      / Output of this help               '."\n";
    print '                                                               '."\n";
    print 'Example of execution:                                          '."\n";
    print '  $ ./make-country-output.pl --aggregate                       '."\n";
    print '  $ ./make-country-output.pl -f4 FILENAME -f6 FILENAME         '."\n";
    print '  $ ./make-country-output.pl -f4 FILENAME -f6 FILENAME --debug '."\n";
    return 0;
}



###
### Main Program
###
## Setting options and arguments
##
# Set of execution options
foreach my $tmp_OPT(@ARGV){
    push(@OPT_ARGS, $tmp_OPT);
    if(($tmp_OPT eq '-a')||($tmp_OPT eq '--aggregate')){ $OPT_AGGR++;  }
    if(($tmp_OPT eq '-d')||($tmp_OPT eq '--debug'    )){ $OPT_DEBUG++; }
    if(($tmp_OPT eq '-h')||($tmp_OPT eq '--help'     )){ $OPT_HELP++;  }
}
if($OPT_DEBUG){
    print "VALUE::OPT_AGGR  $OPT_AGGR\n";
    print "VALUE::OPT_DEBUG $OPT_DEBUG\n";
    print "VALUE::OPT_HELP  $OPT_HELP\n";
    print "-----\n";
}

# Set of write filename
for(my $tmp_COUNT=0; $tmp_COUNT<($#OPT_ARGS+1); $tmp_COUNT++){
    if($OPT_ARGS[$tmp_COUNT] eq '-f4'){ $FILE_IPV4 = $OPT_ARGS[$tmp_COUNT+1]; }
    if($OPT_ARGS[$tmp_COUNT] eq '-f6'){ $FILE_IPV6 = $OPT_ARGS[$tmp_COUNT+1]; }
}
if($OPT_DEBUG){
    print "VALUE::FILE_IPV4 $FILE_IPV4\n";
    print "VALUE::FILE_IPV6 $FILE_IPV6\n";
    print "-----\n";
}

# Execute of initial process or help
if($OPT_HELP){
    $OPT_RETURN = &OUTPUT_SCRIPT_HELP;
}else{
    if(-f "$CIDR_ROOT_DIR/$DATE/$FILE_IPV4"){ unlink "$CIDR_ROOT_DIR/$DATE/$FILE_IPV4"; }
    if(-f "$CIDR_ROOT_DIR/$DATE/$FILE_IPV6"){ unlink "$CIDR_ROOT_DIR/$DATE/$FILE_IPV6"; }
    if($OPT_AGGR){
        unless($OPT_FLAG){ $OPT_FLAG = &SORT_ASCENDING_IPV4; }
        unless($OPT_FLAG){ $OPT_FLAG = &SORT_ASCENDING_IPV6; }
    }
    unless($OPT_FLAG){ $OPT_FLAG = &WRITE_AGGREGATE_IPV4; }
    unless($OPT_FLAG){ $OPT_FLAG = &WRITE_AGGREGATE_IPV6; }
    unless($OPT_FLAG){ $OPT_RETURN = 0; }
}

exit $OPT_RETURN;