#!/usr/bin/perl

use strict;
use warnings;
use utf8;
use File::Path;



###
### Const variable
###
my $OPT_DEBUG   = 0;
my $OPT_FLAG    = 0;
my $OPT_HELP    = 0;
my $OPT_RETURN  = 1;
my @OPT_ARGS;

my $IPLIST_DIR      = "/tmp/$$";
my $IPLIST_OWNER    = "www";
my $IPLIST_GROUP    = "www";
my $IPLIST_PERMIT   = "644";

my $BIN_BZIP2       = "/usr/bin/bzip2";
my $BIN_CHMOD       = "/usr/bin/chmod";
my $BIN_CHOWN       = "/usr/bin/chown";
my $BIN_CP          = "/usr/bin/cp";
my $BIN_GZIP        = "/usr/bin/gzip";
my $BIN_MD5SUM      = "/usr/bin/md5sum";
my $BIN_XZ          = "/usr/bin/xz";

my $WEB_ROOT_PATH   = "/var/www/htdocs";
my $CIDR_ROOT_DIR   = "/var/iplist";
my $FILE_IPV4       = "CIDR-IPV4";
my $FILE_IPV6       = "CIDR-IPV6";

my %IPLIST_IPV4;
$IPLIST_IPV4{'txt'} = "cidr_ipv4.txt";
$IPLIST_IPV4{'md5'} = "cidr_ipv4.md5";
$IPLIST_IPV4{'gz'}  = "cidr_ipv4.gz";
$IPLIST_IPV4{'bz2'} = "cidr_ipv4.bz2";
$IPLIST_IPV4{'xz'}  = "cidr_ipv4.xz";

my %IPLIST_IPV6;
$IPLIST_IPV6{'txt'} = "cidr_ipv6.txt";
$IPLIST_IPV6{'md5'} = "cidr_ipv6.md5";
$IPLIST_IPV6{'gz'}  = "cidr_ipv6.gz";
$IPLIST_IPV6{'bz2'} = "cidr_ipv6.bz2";
$IPLIST_IPV6{'xz'}  = "cidr_ipv6.xz";



###
### Sobroutine
###
## Generate archive file
##
sub CREATE_FILE_ARCHIVE{
    # [txt] Plain Text
    system("$BIN_CP $CIDR_ROOT_DIR/$FILE_IPV4 $IPLIST_DIR/$IPLIST_IPV4{'txt'}");
    system("$BIN_CP $CIDR_ROOT_DIR/$FILE_IPV6 $IPLIST_DIR/$IPLIST_IPV6{'txt'}");

    # [gz]  GZIP
    system("$BIN_GZIP -c $IPLIST_IPV4{'txt'} > $IPLIST_IPV4{'gz'}");
    system("$BIN_GZIP -c $IPLIST_IPV6{'txt'} > $IPLIST_IPV6{'gz'}");

    # [bz2] BZIP2
    system("$BIN_BZIP2 -c $IPLIST_IPV4{'txt'} > $IPLIST_IPV4{'bz2'}");
    system("$BIN_BZIP2 -c $IPLIST_IPV6{'txt'} > $IPLIST_IPV6{'bz2'}");

    # [xz]  XZ Data
    system("$BIN_XZ -cz $IPLIST_IPV4{'txt'} > $IPLIST_IPV4{'xz'}");
    system("$BIN_XZ -cz $IPLIST_IPV6{'txt'} > $IPLIST_IPV6{'xz'}");

    # [md5] IPv4 Hash
    open(WRITE_HASH, ">> $IPLIST_IPV4{'md5'}");
    foreach my $tmp_FILE(sort keys(%IPLIST_IPV4)){
        if($tmp_FILE !~ /md5/){
            print WRITE_HASH `$BIN_MD5SUM $IPLIST_IPV4{$tmp_FILE}`;
        }
    }
    close(WRITE_HASH);

    # [md5] IPv6 Hash
    open(WRITE_HASH, ">> $IPLIST_IPV6{'md5'}");
    foreach my $tmp_FILE(sort keys(%IPLIST_IPV6)){
        if($tmp_FILE !~ /md5/){
            print WRITE_HASH `$BIN_MD5SUM $IPLIST_IPV6{$tmp_FILE}`;
        }
    }
    close(WRITE_HASH);

    # Debug 
    if($OPT_DEBUG){
        open(READ_HASH, "$WEB_ROOT_PATH/$IPLIST_IPV4{'md5'}");
        foreach(<READ_HASH>){ print "$_"; }
        close(READ_HASH);
        print "-----\n";

        open(READ_HASH, "$WEB_ROOT_PATH/$IPLIST_IPV6{'md5'}");
        foreach(<READ_HASH>){ print "$_"; }
        close(READ_HASH);
        print "-----\n";
    }
    return 0;
}


## Copy ip list to web root
##
sub COPY_WEB_ROOT{
    if(-e "$WEB_ROOT_PATH"){
        foreach my $tmp_FILE(sort keys(%IPLIST_IPV4)){
            if(-f "$WEB_ROOT_PATH/$tmp_FILE"){
                unlink "$WEB_ROOT_PATH/$tmp_FILE";
            }
        }
        foreach my $tmp_FILE(sort keys(%IPLIST_IPV6)){
            if(-f "$WEB_ROOT_PATH/$tmp_FILE"){
                unlink "$WEB_ROOT_PATH/$tmp_FILE";
            }
        }
        foreach my $tmp_FILE(glob("$IPLIST_DIR/*")){
            $tmp_FILE =~ s/.*\///g;
            system("$BIN_CP    $IPLIST_DIR/$tmp_FILE       $WEB_ROOT_PATH/         ");
            system("$BIN_CHOWN $IPLIST_OWNER.$IPLIST_GROUP $WEB_ROOT_PATH/$tmp_FILE");
            system("$BIN_CHMOD $IPLIST_PERMIT              $WEB_ROOT_PATH/$tmp_FILE");
        }
    }
    return 0;
}


## Output help
##
sub OUTPUT_SCRIPT_HELP{
    print 'Usage: make-country-webdir.pl [ OPTIONS ]         '."\n";
    print '                                                  '."\n";
    print 'OPTIONS := -d[--debug] / Standard stream for debug'."\n";
    print '           -h[--help]  / Output of this help      '."\n";
    print '                                                  '."\n";
    print 'Example of execution:                             '."\n";
    print '  $ ./make-country-webdir.pl                      '."\n";
    print '  $ ./make-country-webdir.pl --debug              '."\n";
    return 0;
}



###
### Main Program
###
## Setting options and arguments
##
# Set of execution options
foreach my $tmp_OPT(@ARGV){
    push(@OPT_ARGS, $tmp_OPT);
    if(($tmp_OPT eq '-d')||($tmp_OPT eq '--debug')){ $OPT_DEBUG++; }
    if(($tmp_OPT eq '-h')||($tmp_OPT eq '--help' )){ $OPT_HELP++;  }
}
if($OPT_DEBUG){
    print "VALUE::OPT_DEBUG $OPT_DEBUG\n";
    print "VALUE::OPT_HELP  $OPT_HELP\n";
    print "-----\n";
}

# Execute of initial process or help
if($OPT_HELP){
    $OPT_RETURN = &OUTPUT_SCRIPT_HELP;
}else{
    if(-e "$IPLIST_DIR"){ rmtree("$IPLIST_DIR"); }
    mkdir $IPLIST_DIR, 0755;
    chdir $IPLIST_DIR;
    unless($OPT_FLAG){ $OPT_FLAG = &CREATE_FILE_ARCHIVE; }
    unless($OPT_FLAG){ $OPT_FLAG = &COPY_WEB_ROOT;       }
    unless($OPT_FLAG){ $OPT_RETURN = 0;                  }

    # Delete working directory
    chdir "/tmp";
    rmtree("$IPLIST_DIR");
}

exit $OPT_RETURN;